/*
* Simulation model for the ad7689 IC
* General timing without a busy indicator
* */

`timescale 1 ns / 100 ps
module ad7689_model(
    input           din_i,
    input           sck_i,
    input           cnv_i,

    output          sdo_o
);

// State Machine - State Names (parameters)
parameter IDLE              = 1;    // Initial Power Up State
parameter CONVERSION        = 2;
parameter QUIET             = 3;
parameter ACQUISITION       = 4;

integer state;                      // state

reg [15:0] data;        // data register of the ADC
reg [16:0] new_data;    // register used for new data acquirements. Updated at the end of the QUIET period
reg [15:0] cfg;         // configuration register of the ADC

initial
begin
    state           = IDLE;
    new_data        = 16'h8000;
    data            = 16'h8000;
    cfg             = 16'h0000;
end

always @(posedge cnv_i) // state machine , initiateb by activation of CNV
begin
        state = CONVERSION;
        #1200   //tdata
        state = QUIET;
        data        <= data + 16'b1;
        new_data    <= data;
        #1000
        state = ACQUISITION;
end

assign sdo_o    = new_data [16];

always @(posedge sck_i)
begin
    case(state)
        IDLE:
        begin
            $display ("SCLK has been activate in IDLE state. This should not happen");
        end
        CONVERSION:
        begin
            new_data    <= {new_data [15:0], new_data[16]};
        end
        QUIET:
        begin
            $display ("SCLK has been activate in QUIET state. This should not happen");
        end
        ACQUISITION:
        begin
            new_data    <= { new_data [15:0], new_data[15] };
        end
    endcase
end

endmodule
